﻿using System;
using System.Windows.Forms;
using System.IO;
using System.Drawing;
using System.Collections.Generic;
using STFXmlParser;

namespace CalcHelper
{
    public partial class Form1 : Form
    {
        public string GetFileName(string pdfId)
        {
            string fileName = null;
            OpenFileDialog ofd = new OpenFileDialog();

            switch (pdfId)
            {
                case ("pdf1"):

                    ofd.InitialDirectory = pdf1CurDir;

                    break;

                case ("pdf2"):

                    ofd.InitialDirectory = pdf2CurDir;

                    break;
            }

            ofd.Filter = "Zip File|*.zip";

            System.Windows.Forms.DialogResult dr = ofd.ShowDialog();

            if (dr == DialogResult.OK)
            {
                // This holds the path and the file name with extension.
                fileName = ofd.FileName;
            }

            return fileName;
        }

        public void OpenFilePreProcess(string pdfId)
        {
            string fileName;

            fileName = GetFileName(pdfId);

            if (!string.IsNullOrEmpty(fileName))
            {
                switch (pdfId)
                {
                    case ("pdf1"):

                        if (pdf1 != null)
                        {
                            ProcessFileClose(ref pdf1, ref pa1);
                        }

                        pdf1 = new ProcessPdf("pdf1", ref panel1, ref pictureBox1);

                        OpenFile(ref pdf1, ref pa1, fileName);

                        break;

                    case ("pdf2"):

                        if (pdf2 != null)
                        {
                            ProcessFileClose(ref pdf2, ref pa2);
                        }

                        pdf2 = new ProcessPdf("pdf2", ref panel2, ref pictureBox2);

                        OpenFile(ref pdf2, ref pa2, fileName);

                        break;
                }
            }
        }

        public void OpenFile(ref ProcessPdf pdf, ref PageArray pa, string fileName)
        {
            if (pdf == null)
            {
                MessageBox.Show("OpenFile() received a null pdf argument.", "null ProcessPdf argument:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (fileName.ToLower().EndsWith(".zip"))
            {
                if (pdf.ExtractFiles(fileName))
                {
                    switch (pdf.id)
                    {
                        case ("pdf1"):

                            this.Text = Path.GetFileName(fileName) + " - Calc Helper 1.0";
                            docLabel.Text = fileName;

                            pdf1CurDir = Path.GetDirectoryName(fileName);

                            break;

                        case ("pdf2"):

                            docLabel2.Text = fileName;

                            pdf2CurDir = Path.GetDirectoryName(fileName);

                            break;
                    }

                    if (pdf.GetXmlData(ref pa))
                    {
                        if (pdf.ScaleRects("OpenFile", ref pa))
                        {
                            if (pdf.numPages > 0)
                            {
                                ProcessFileOpen(ref pdf, ref pa, false);
                            }
                        }
                    }
                }
                else
                {
                    MessageBox.Show("Application needs BOTH the .pdf and the .xml [boxes] files", string.Format("{0} - Application can not process {1}:", pdf.id, Path.GetFileName(fileName)), MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
            }
        }

        public void ProcessPageNum(ref ProcessPdf pdf)
        {
            if (pdf == null)
            {
                MessageBox.Show("ProcessPageNum() received a null pdf argument.", "null ProcessPdf argument:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            /*
             * Enable/disable the page navigation toolstrip buttons to reflect
             * current page as it relates to the page navigation mode.
             * 
             * Note that pdf.curPage still holds the current page regardless of
             * the page navigation mode, but we need to look at the pdf.boxedPageList
             * index value when in Boxed Pages mode.
            */

            int curVal, upperVal, lowerVal;

            if (pdf.pageNav)
            {
                curVal = (int)pdf.curPage;
                upperVal = (int)pdf.numPages;
                lowerVal = 1;
            }
            else
            {
                curVal = pdf.boxedPageIndex;
                upperVal = (pdf.boxedPageList.Count - 1);
                lowerVal = 0;
            }

            // Each split screen has a plethora of menu buttons:
            switch (pdf.id)
            {
                case ("pdf1"):

                    if (curVal < upperVal)
                    {
                        if (!lastPageBtn.Enabled)
                        {
                            nextPageBtn.Enabled = true;
                            nextPageToolStripMenuItem.Enabled = true;

                            lastPageBtn.Enabled = true;
                            lastPageToolStripMenuItem.Enabled = true;
                        }
                    }
                    else
                    {
                        nextPageBtn.Enabled = false;
                        nextPageToolStripMenuItem.Enabled = false;

                        lastPageBtn.Enabled = false;
                        lastPageToolStripMenuItem.Enabled = false;
                    }

                    if (curVal > lowerVal)
                    {
                        if (!firstPageBtn.Enabled)
                        {
                            prevPageBtn.Enabled = true;
                            previousPageToolStripMenuItem.Enabled = true;

                            firstPageBtn.Enabled = true;
                            firstPageToolStripMenuItem.Enabled = true;
                        }
                    }
                    else
                    {
                        prevPageBtn.Enabled = false;
                        previousPageToolStripMenuItem.Enabled = false;

                        firstPageBtn.Enabled = false;
                        firstPageToolStripMenuItem.Enabled = false;
                    }

                    if (pdf.numPages > 1)
                    {
                        goToPageToolStripMenuItem.Enabled = true;
                    }
                    else
                    {
                        goToPageToolStripMenuItem.Enabled = false;
                    }

                    pageLabel.Text = string.Format("Page {0} of {1}", pdf.curPage, pdf.numPages);

                    break;

                case ("pdf2"):

                    if (curVal < upperVal)
                    {
                        if (!lastPageBtn2.Enabled)
                        {
                            nextPageBtn2.Enabled = true;
                            nextPageToolStripMenuItem2.Enabled = true;

                            lastPageBtn2.Enabled = true;
                            lastPageToolStripMenuItem2.Enabled = true;
                        }
                    }
                    else
                    {
                        nextPageBtn2.Enabled = false;
                        nextPageToolStripMenuItem2.Enabled = false;

                        lastPageBtn2.Enabled = false;
                        lastPageToolStripMenuItem2.Enabled = false;
                    }

                    if (curVal > lowerVal)
                    {
                        if (!firstPageBtn2.Enabled)
                        {
                            prevPageBtn2.Enabled = true;
                            previousPageToolStripMenuItem2.Enabled = true;

                            firstPageBtn2.Enabled = true;
                            firstPageToolStripMenuItem2.Enabled = true;
                        }
                    }
                    else
                    {
                        prevPageBtn2.Enabled = false;
                        previousPageToolStripMenuItem2.Enabled = false;

                        firstPageBtn2.Enabled = false;
                        firstPageToolStripMenuItem2.Enabled = false;
                    }

                    if (pdf.numPages > 1)
                    {
                        goToPageToolStripMenuItem2.Enabled = true;
                    }
                    else
                    {
                        goToPageToolStripMenuItem2.Enabled = false;
                    }

                    pageLabel2.Text = string.Format("Page {0} of {1}", pdf.curPage, pdf.numPages);

                    break;
            }
        }

        public void ProcessPageNavMode(Boolean pageMode, string pdfId)
        {
            switch (pdfId)
            {
                case ("pdf1"):

                    if (pageMode)
                    {
                        // Navigate to any document page, strictly linear.

                        pageNavigateModeBtn.ToolTipText = "Navigate: To Any Page";
                        pageNavigateModeBtn.Text = "P";

                        firstPageBtn.ToolTipText = "First Page";
                        firstPageToolStripMenuItem.Text = "First Page";

                        prevPageBtn.ToolTipText = "Previous Page";
                        previousPageToolStripMenuItem.Text = "Previous Page";

                        nextPageBtn.ToolTipText = "Next Page";
                        nextPageToolStripMenuItem.Text = "Next Page";

                        lastPageBtn.ToolTipText = "Last Page";
                        lastPageToolStripMenuItem.Text = "Last Page";
                    }
                    else
                    {
                        // Navigate to just pages with boxes.

                        pageNavigateModeBtn.ToolTipText = "Navigate: To Just Pages With Boxes";
                        pageNavigateModeBtn.Text = "B";

                        firstPageBtn.ToolTipText = "First Boxed Page";
                        firstPageToolStripMenuItem.Text = "First Boxed Page";

                        prevPageBtn.ToolTipText = "Previous Boxed Page";
                        previousPageToolStripMenuItem.Text = "Previous Boxed Page";

                        nextPageBtn.ToolTipText = "Next Boxed Page";
                        nextPageToolStripMenuItem.Text = "Next Boxed Page";

                        lastPageBtn.ToolTipText = "Last Boxed Page";
                        lastPageToolStripMenuItem.Text = "Last Boxed Page";
                    }
                    break;

                case ("pdf2"):

                    if (pageMode)
                    {
                        // Navigate to any document page, strictly linear.

                        pageNavigateModeBtn2.ToolTipText = "Navigate: To Any Page";
                        pageNavigateModeBtn2.Text = "P";

                        firstPageBtn2.ToolTipText = "First Page";
                        firstPageToolStripMenuItem2.Text = "First Page";

                        prevPageBtn2.ToolTipText = "Previous Page";
                        previousPageToolStripMenuItem2.Text = "Previous Page";

                        nextPageBtn2.ToolTipText = "Next Page";
                        nextPageToolStripMenuItem2.Text = "Next Page";

                        lastPageBtn2.ToolTipText = "Last Page";
                        lastPageToolStripMenuItem2.Text = "Last Page";
                    }
                    else
                    {
                        // Navigate to just pages with boxes.

                        pageNavigateModeBtn2.ToolTipText = "Navigate: To Just Pages With Boxes";
                        pageNavigateModeBtn2.Text = "B";

                        firstPageBtn2.ToolTipText = "First Boxed Page";
                        firstPageToolStripMenuItem2.Text = "First Boxed Page";

                        prevPageBtn2.ToolTipText = "Previous Boxed Page";
                        previousPageToolStripMenuItem2.Text = "Previous Boxed Page";

                        nextPageBtn2.ToolTipText = "Next Boxed Page";
                        nextPageToolStripMenuItem2.Text = "Next Boxed Page";

                        lastPageBtn2.ToolTipText = "Last Boxed Page";
                        lastPageToolStripMenuItem2.Text = "Last Boxed Page";
                    }

                    break;
            }
        }

        public void ProcessFileClose(ref ProcessPdf pdf, ref PageArray pa)
        {
            if (pdf == null)
            {
                MessageBox.Show("ProcessFileClose() received a null pdf argument.", "null ProcessPdf argument:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            /*
             * Remove previously extracted files and the directory that holds them,
             * BUT we first need to check if the view is currently split. If it is, we
             * will not cleanup the files if pdf2 has the same extrusion path as pdf1.
             * pdf2 will cleanup those files when either it opens a new file, or we go
             * to single view, or we close the form.
            */
            bool doCleanup = true;

            pdf.ClearBmaps();

            pdf.panel.BackColor = SystemColors.ControlDark;

            DisableZoomResetBtns(ref pdf);

            pa = null;

            switch (pdf.id)
            {
                case ("pdf1"):

                    if (pdf2 != null)
                    {
                        if (pdf2.ExtPath == pdf.ExtPath)
                        {
                            doCleanup = false;
                        }
                    }

                    if (doCleanup)
                    {
                        pdf.CleanupFiles();
                    }

                    closeFileBtn.Enabled = false;

                    if (splitScreen)
                    {
                        copyPaneToolStripMenuItem2.Enabled = false;
                    }
                    else
                    {
                        singleScrnBtn.Enabled = false;
                        singleScreenToolStripMenuItem.Enabled = false;

                        splitHorizBtn.Enabled = false;
                        splitScreenHorizontallyToolStripMenuItem.Enabled = false;

                        splitVertBtn.Enabled = false;
                        splitScreenVerticallyToolStripMenuItem.Enabled = false;
                    }

                    firstPageBtn.Enabled = false;
                    firstPageToolStripMenuItem.Enabled = false;

                    prevPageBtn.Enabled = false;
                    previousPageToolStripMenuItem.Enabled = false;

                    nextPageBtn.Enabled = false;
                    nextPageToolStripMenuItem.Enabled = false;

                    lastPageBtn.Enabled = false;
                    lastPageToolStripMenuItem.Enabled = false;

                    goToPageToolStripMenuItem.Enabled = false;
                    pageNavigateModeBtn.Enabled = false;

                    zoomInBtn.Enabled = false;
                    zoomInToolStripMenuItem.Enabled = false;

                    zoomOutBtn.Enabled = false;
                    zoomOutToolStripMenuItem.Enabled = false;

                    fullWidthBtn.Enabled = false;
                    fullWidthToolStripMenuItem.Enabled = false;

                    fullPageBtn.Enabled = false;
                    fullPageToolStripMenuItem.Enabled = false;

                    this.Text = "Calc Helper 1.0";
                    docLabel.Text = "";
                    pageLabel.Text = "";

                    pdf1 = null;

                    break;

                case ("pdf2"):

                    if (pdf1 != null)
                    {
                        if (pdf1.ExtPath == pdf.ExtPath)
                        {
                            doCleanup = false;
                        }
                    }

                    if (doCleanup)
                    {
                        pdf.CleanupFiles();
                    }

                    closeFileBtn2.Enabled = false;
                    closeToolStripMenuItem2.Enabled = false;

                    firstPageBtn2.Enabled = false;
                    firstPageToolStripMenuItem2.Enabled = false;

                    prevPageBtn2.Enabled = false;
                    previousPageToolStripMenuItem2.Enabled = false;

                    nextPageBtn2.Enabled = false;
                    nextPageToolStripMenuItem2.Enabled = false;

                    lastPageBtn2.Enabled = false;
                    lastPageToolStripMenuItem2.Enabled = false;

                    goToPageToolStripMenuItem2.Enabled = false;
                    pageNavigateModeBtn2.Enabled = false;

                    zoomInBtn2.Enabled = false;
                    zoomInToolStripMenuItem2.Enabled = false;

                    zoomOutBtn2.Enabled = false;
                    zoomOutToolStripMenuItem2.Enabled = false;

                    fullWidthBtn2.Enabled = false;
                    fullWidthToolStripMenuItem2.Enabled = false;

                    fullPageBtn2.Enabled = false;
                    fullPageToolStripMenuItem2.Enabled = false;

                    docLabel2.Text = "";
                    pageLabel2.Text = "";

                    pdf2 = null;

                    break;
            }
        }

        public void EnableZoomResetBtns(ref ProcessPdf pdf)
        {
            if (pdf == null)
            {
                MessageBox.Show("EnableZoomResetBtns() received a null pdf argument.", "null ProcessPdf argument:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            switch (pdf.id)
            {
                case ("pdf1"):

                    if (!zoomResetBtn.Enabled)
                    {
                        zoomResetBtn.Image = Properties.Resources.ZoomResetOn;
                        zoomResetBtn.Enabled = true;
                        zoomResetToolStripMenuItem.Enabled = true;
                    }

                    break;

                case ("pdf2"):

                    if (!zoomResetBtn2.Enabled)
                    {
                        zoomResetBtn2.Image = Properties.Resources.ZoomResetOn;
                        zoomResetBtn2.Enabled = true;
                        zoomResetToolStripMenuItem2.Enabled = true;
                    }

                    break;
            }
        }

        public void DisableZoomResetBtns(ref ProcessPdf pdf)
        {
            if (pdf == null)
            {
                MessageBox.Show("DisableZoomResetBtns() received a null pdf argument.", "null ProcessPdf argument:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            switch (pdf.id)
            {
                case ("pdf1"):

                    if (zoomResetBtn.Enabled)
                    {
                        zoomResetBtn.Image = Properties.Resources.ZoomResetOff;
                        zoomResetBtn.Enabled = false;
                        zoomResetToolStripMenuItem.Enabled = false;
                    }

                    break;

                case ("pdf2"):

                    if (zoomResetBtn2.Enabled)
                    {
                        zoomResetBtn2.Image = Properties.Resources.ZoomResetOff;
                        zoomResetBtn2.Enabled = false;
                        zoomResetToolStripMenuItem2.Enabled = false;
                    }

                    break;
            }
        }

        public void ProcessFileOpen(ref ProcessPdf pdf, ref PageArray pa, bool samePage)
        {
            /*
             * When samePage is true:
             * 
             * 1) We are displaying a pdf in the pdf1 panel and are splitting the screen
             *    either vertically or horizontally. We are going to display the same pdf1
             *    pdf in the pdf2 panel and are going to display the same current page and
             *    also have the same menu button enabled/disabled values.
             * 
             * 2) We have a split screen display and the user wants to switch to a single
             *    screen display. The pdf1 panel is empty and the pdf2 panel is not. We will
             *    display the pdf2 pdf in the pdf1 panel and will also use the same current
             *    page and menu button enabled/disabled statuses.
            */
            if (pdf == null)
            {
                MessageBox.Show("ProcessFileOpen() received a null pdf argument.", "null ProcessPdf argument:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (pa == null)
            {
                MessageBox.Show("ProcessFileOpen() received a null pa argument.", "null PageArray argument:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            pdf.panel.BackColor = SystemColors.Control;

            // Set pdf.pageList - used in boxed page navigation mode.
            pdf.boxedPageList = pa.GetBoxedPageList();

            // Check if a samePage copy:
            if (samePage)
            {
                switch (pdf.id)
                {
                    case ("pdf1"):

                        if (pdf2 != null)
                        {
                            pdf.pageNav = pdf2.pageNav;
                            pdf.boxedPageIndex = pdf2.boxedPageIndex;
                            pdf.curPage = pdf2.curPage;
                        }

                        break;

                    case ("pdf2"):

                        if (pdf1 != null)
                        {
                            pdf.pageNav = pdf1.pageNav;
                            pdf.boxedPageIndex = pdf1.boxedPageIndex;
                            pdf.curPage = pdf1.curPage;
                        }

                        break;
                }
            }
            else
            {
                // Auto-select the Page/Boxed Page navigation mode:
                if (pdf.numPages != pdf.boxedPageList.Count)
                {
                    // Switch page navigation to boxed pages:
                    pdf.pageNav = false;
                    pdf.boxedPageIndex = 0;
                    pdf.curPage = pdf.boxedPageList[0];
                }
                else
                {
                    // Set page navigation to pages:
                    pdf.pageNav = true;
                    pdf.curPage = 1;
                }
            }

            /*
             * This sets the text on the first/previous/next/last/page
             * navigation buttons. For example text will either read 
             * like 'navigate to next page' or 'navigate to next boxed page'.
            */
            ProcessPageNavMode(pdf.pageNav, pdf.id);

            pdf.ShowPage((int)pdf.curPage, true, ref pa);

            // This handles page navigation menu items/buttons being active/inactive.
            ProcessPageNum(ref pdf);

            /*
             * Enable non-page navigation and non-Zoom Reset menu items/buttons.
             * Zoom Reset and page navigation menu buttons/items
             * are handled by other logic associated with them.
             * 
             * If we are opening a file for pdf1 we check if the screen is split.
             * If the second screen panel is null [not occupied by a pdf] we will
             * copy the pdf1 pdf over to the second panel which we refer to as pdf2.
            */
            switch (pdf.id)
            {
                case ("pdf1"):

                    closeFileBtn.Enabled = true;

                    zoomInBtn.Enabled = true;
                    zoomInToolStripMenuItem.Enabled = true;

                    zoomOutBtn.Enabled = true;
                    zoomOutToolStripMenuItem.Enabled = true;

                    fullWidthBtn.Enabled = true;
                    fullWidthToolStripMenuItem.Enabled = true;

                    fullPageBtn.Enabled = true;
                    fullPageToolStripMenuItem.Enabled = true;

                    pageNavigateModeBtn.Enabled = true;

                    if (splitScreen)
                    {
                        copyPaneToolStripMenuItem2.Enabled = true;

                        if (pdf2 == null)
                        {
                            CopyPdf1ToPdf2(false);
                        }
                    }
                    else
                    {
                        splitHorizBtn.Enabled = true;
                        splitScreenHorizontallyToolStripMenuItem.Enabled = true;

                        splitVertBtn.Enabled = true;
                        splitScreenVerticallyToolStripMenuItem.Enabled = true;
                    }

                    break;

                case ("pdf2"):

                    closeFileBtn2.Enabled = true;
                    closeToolStripMenuItem2.Enabled = true;

                    zoomInBtn2.Enabled = true;
                    zoomInToolStripMenuItem2.Enabled = true;

                    zoomOutBtn2.Enabled = true;
                    zoomOutToolStripMenuItem2.Enabled = true;

                    fullWidthBtn2.Enabled = true;
                    fullWidthToolStripMenuItem2.Enabled = true;

                    fullPageBtn2.Enabled = true;
                    fullPageToolStripMenuItem2.Enabled = true;

                    pageNavigateModeBtn2.Enabled = true;

                    break;
            }
        }

        public void CopyPdf1ToPdf2(bool samePage)
        {
            /*
             * We are splitting the screen and we want to show the same pdf
             * in the pdf2 panel [hence the name]. Argument 'samePage' is 
             * passed on to ProcessFileOpen(), when true it will display the
             * same pdf1 page in the pdf2 panel.
            */
            if (pdf1 == null)
            {
                MessageBox.Show("CopyPdf1ToPdf2() detected a null pdf1, copy aborted.", "null pdf1 detected:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            /*
             * The caller ensures pdf2 is null, as we do not want to
             * overwrite another pdf.
            */
            pdf2 = new ProcessPdf("pdf2", ref panel2, ref pictureBox2);

            pdf2.pdfFile = pdf1.pdfFile;
            pdf2.xmlFile = pdf1.xmlFile;
            pdf2.ExtPath = pdf1.ExtPath;

            pdf2CurDir = pdf1CurDir;

            docLabel2.Text = docLabel.Text;

            // We are reading the same xmlFile as used by pdf1,
            // making our own distinct copy in pa2.
            if (pdf2.GetXmlData(ref pa2))
            {
                if (pdf2.ScaleRects("OpenFile", ref pa2))
                {
                    if (pdf2.numPages > 0)
                    {
                        ProcessFileOpen(ref pdf2, ref pa2, samePage);
                    }
                }
            }
        }

        public void CopyPdf2ToPdf1(bool samePage)
        {
            /*
             * We are switching to a single screen view, the pdf1 panel
             * is not displaying a pdf, the pdf2 panel is displaying a
             * pdf, so we are going to display the pdf2's pdf in the
             * pdf1's panel.
             * 
             * Argument 'samePage' is passed on to ProcessFileOpen(), when
             * true it will display the same pdf2 page in the pdf1 panel,
             * same menu buttons enabled/disabled.
             * 
             * To the user it will be pdf2 displayed as a full page but we
             * are transferring it to pdf1. I want to keep the 'main' [what
             * you see when you first start the program] panel as pdf1 and
             * the 'new' [when they split the screen] panel as pdf2. It is
             * pdf2's panel that gets collapsed/un-collapsed.
             * 
             * *****************************************************
             * 
             * IMPORTANT FUTURE NOTES:
             * 
             * WE WILL NEED TO CHECK IF THE pdf2 FILE IS COUPLED WITH THE 
             * TOOL'S CALCULATION LOGIC. IF IT IS NOT WE NEED TO PROMPT THE
             * USER IF THEY WANT THIS FILE TO BE COUPLED TO THE TOOL. THE
             * FILE COULD BE A PREVIOUS VERSION OF THE PDF1 FILE AND THEY
             * HAD IT OPEN FOR JUST VIEWING OR WHATEVER. OR SOMETHING LIKE
             * THIS - THIS WILL NEED MORE CONSIDERATION.
             * 
             * WILL PROBABLY HAVE THE TOOL INCORPORATE THE PDF2'S FILE WHEN IT
             * IS THE SAME FILE AS THE PDF1 FILE - IN THIS CASE THEY SPLIT THE
             * SCREEN TO MORE EASILY POST BETWEEN PAGES, LOOK AT RELATED CALCS,
             * HANDLE CALCS THAT SPAN PAGES, ETC.. THEY CURRENTLY CAN ALSO OPEN
             * THE PDF1 FILE FROM THE PDF2 PANEL.
             * 
             * *****************************************************
            */
            if (pdf2 == null)
            {
                MessageBox.Show("CopyPdf2ToPdf1() detected a null pdf2, copy aborted.", "null pdf2 detected:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            /*
             * The caller ensures pdf1 is null, as we do not want to
             * overwrite another pdf.
            */
            pdf1 = new ProcessPdf("pdf1", ref panel1, ref pictureBox1);

            pdf1.pdfFile = pdf2.pdfFile;
            pdf1.xmlFile = pdf2.xmlFile;
            pdf1.ExtPath = pdf2.ExtPath;

            pdf1CurDir = pdf2CurDir;

            docLabel.Text = docLabel2.Text;

            // We are reading the same xmlFile as used by pdf2,
            // making our own distinct copy in pa1.
            if (pdf1.GetXmlData(ref pa1))
            {
                if (pdf1.ScaleRects("OpenFile", ref pa1))
                {
                    if (pdf1.numPages > 0)
                    {
                        ProcessFileOpen(ref pdf1, ref pa1, samePage);
                    }
                }
            }
        }

        public void ProcessZoomIn(ref ProcessPdf pdf, ref PageArray pa)
        {
            if (pdf == null)
            {
                MessageBox.Show("ProcessZoomIn() received a null pdf argument.", "null ProcessPdf argument:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (pa == null)
            {
                MessageBox.Show("ProcessZoomIn() received a null pa argument.", "null PageArray argument:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            PgZoomData zd = null;

            /*
             * First check pdf.initZoomReset. If true, we initialize the page dictionary 
             * with the dpi and page bitmap heights and widths per page. We roll-back to
             * these settings for a zoom reset operation.
            */
            if (pdf.initZoomReset)
            {
                pdf.SetZoomResets(ref pdf, ref pa);
            }

            pdf.ScaleRects("ZoomIn", ref pa);

            if (pdf.curPageZoomDelta)
            {
                pdf.curPageZoomDelta = false;

                pdf.vScrollPos = (int)((double)pdf.panel.VerticalScroll.Value * pdf.zoomInVal);
                pdf.hScrollPos = (int)((double)pdf.panel.HorizontalScroll.Value * pdf.zoomInVal);

                pdf.ShowPage((int)pdf.curPage, true, ref pa);

                pa.GetPgZoomDataRef((int)pdf.curPage - 1, ref zd);

                if (zd == null)
                {
                    MessageBox.Show(string.Format("pa.GetPgZoomDataRef({0}, ref zd) has returned a null PgZoomData.", pdf.curPage - 1), "ProcessZoomIn()", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }

                /*
                 * We now disable or enable if not enabled already, the zoom reset
                 * buttons for the current page:
                */
                if (zd.zoomCnt == 0)
                {
                    /*
                     * We had prior 'zoom outs' which drove the page's zoom count to
                     * a negative value. This zoom in brought the count up to zero.
                    */
                    DisableZoomResetBtns(ref pdf);
                }
                else
                {
                    EnableZoomResetBtns(ref pdf);
                }

                /*
                 * Trying to set scrollbars in same relative position,
                 * relative to upper left corner of PicPanel.
                 * Need to set these twice for some reason.
                */
                pdf.panel.VerticalScroll.Value = pdf.vScrollPos;
                pdf.panel.VerticalScroll.Value = pdf.vScrollPos;

                pdf.panel.HorizontalScroll.Value = pdf.hScrollPos;
                pdf.panel.HorizontalScroll.Value = pdf.hScrollPos;
            }
        }

        public void ProcessZoomOut(ref ProcessPdf pdf, ref PageArray pa)
        {
            if (pdf == null)
            {
                MessageBox.Show("ProcessZoomOut() received a null pdf argument.", "null ProcessPdf argument:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (pa == null)
            {
                MessageBox.Show("ProcessZoomOut() received a null pa argument.", "null PageArray argument:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            PgZoomData zd = null;

            /*
             * First check pdf.initZoomReset. If true, we initialize the page dictionary 
             * with the dpi and page bitmap heights and widths per page. We roll-back to
             * these settings for a zoom reset operation.
            */
            if (pdf.initZoomReset)
            {
                pdf.SetZoomResets(ref pdf, ref pa);
            }

            pdf.ScaleRects("ZoomOut", ref pa);

            if (pdf.curPageZoomDelta)
            {
                pdf.curPageZoomDelta = false;

                // Current pre-zoom out values.
                pdf.vScrollPos = pdf.panel.VerticalScroll.Value;
                pdf.hScrollPos = pdf.panel.HorizontalScroll.Value;

                pdf.ShowPage((int)pdf.curPage, true, ref pa);

                pa.GetPgZoomDataRef((int)pdf.curPage - 1, ref zd);

                if (zd == null)
                {
                    MessageBox.Show(string.Format("pa.GetPgZoomDataRef({0}, ref zd) has returned a null PgZoomData.", pdf.curPage - 1), "ProcessZoomOut()", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }

                /*
                 * We now disable or enable if not enabled already, the zoom reset
                 * buttons for the current page:
                */
                if (zd.zoomCnt == 0)
                {
                    /*
                     * We had prior 'zoom ins' which drove the page's zoom count to
                     * a positive value. This zoom out brought the count down to zero.
                    */
                    DisableZoomResetBtns(ref pdf);
                }
                else
                {
                    EnableZoomResetBtns(ref pdf);
                }

                /*
                 * Now zoom-out scrollbar position(s), if still there,
                 * trying to keep them in same relative position(s):
                 * 
                 * Again, we need to set positions twice for some reason.
                */
                if (pdf.bMap.Width > (pdf.panel.Width - 2))
                {
                    pdf.hScrollPos = (int)((double)pdf.hScrollPos * pdf.zoomOutVal);

                    pdf.panel.HorizontalScroll.Value = pdf.hScrollPos;
                    pdf.panel.HorizontalScroll.Value = pdf.hScrollPos;
                }

                if (pdf.bMap.Height > (pdf.panel.Height - 2))
                {
                    pdf.vScrollPos = (int)((double)pdf.vScrollPos * pdf.zoomOutVal);

                    pdf.panel.VerticalScroll.Value = pdf.vScrollPos;
                    pdf.panel.VerticalScroll.Value = pdf.vScrollPos;
                }
            }
        }

        public void ProcessZoomReset(ref ProcessPdf pdf, ref PageArray pa)
        {
            if (pdf == null)
            {
                MessageBox.Show("ProcessZoomReset() received a null pdf argument.", "null ProcessPdf argument:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (pa == null)
            {
                MessageBox.Show("ProcessZoomReset() received a null pa argument.", "null PageArray argument:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            PgZoomData zd = null;
            int zoomCnt;

            // Get the current page's zoom count before we reset.
            pa.GetPgZoomDataRef((int)pdf.curPage - 1, ref zd);

            if (zd == null)
            {
                MessageBox.Show(string.Format("pa.GetPgZoomDataRef({0}, ref zd) has returned a null PgZoomData.\n\nAborting Zoom Reset operation.", pdf.curPage - 1), "ProcessZoomReset()", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            zoomCnt = zd.zoomCnt;

            pdf.ScaleRects("ZoomReset", ref pa);

            /*
             * This will attempt to keep any scrollbars in same relative
             * locations after resetting the zoom.
            */

            // Current pre-zoom out values.
            pdf.vScrollPos = pdf.panel.VerticalScroll.Value;
            pdf.hScrollPos = pdf.panel.HorizontalScroll.Value;

            pdf.ShowPage((int)pdf.curPage, true, ref pa);

            /*
             * Now reset scrollbar position(s), if still there,
             * trying to keep them in same relative position(s):
             * 
             * Again, we need to set positions twice for some reason.
             * We also need to pay attention to the zoomCnt sign,
             * a negative value means we need to zoom in the value
             * while a positive means we zoom out the scroll values.
             * 
             * Note that this is an approximation, especially when
             * this page has been zoomed out to the point of no
             * scroll bars in which case we 'forced' the image to 
             * either a full page or a full width so we lose the
             * connection between the zoomIn and zoomOut values i.e.
             * A x 1.25 = C but C * 0.8 no longer gives you A, and
             * vice versa.
            */
            if (pdf.bMap.Width > (pdf.panel.Width - 2))
            {
                if (zoomCnt < 0)
                {
                    pdf.hScrollPos = (int)((double)pdf.hScrollPos * Math.Pow(pdf.zoomInVal, (double)Math.Abs(zoomCnt)));
                }
                else
                {
                    pdf.hScrollPos = (int)((double)pdf.hScrollPos * Math.Pow(pdf.zoomOutVal, (double)zoomCnt));
                }

                pdf.panel.HorizontalScroll.Value = pdf.hScrollPos;
                pdf.panel.HorizontalScroll.Value = pdf.hScrollPos;
            }

            if (pdf.bMap.Height > (pdf.panel.Height - 2))
            {
                if (zoomCnt < 0)
                {
                    pdf.vScrollPos = (int)((double)pdf.vScrollPos * Math.Pow(pdf.zoomInVal, (double)Math.Abs(zoomCnt)));
                }
                else
                {
                    pdf.vScrollPos = (int)((double)pdf.vScrollPos * Math.Pow(pdf.zoomOutVal, (double)zoomCnt));
                }

                pdf.panel.VerticalScroll.Value = pdf.vScrollPos;
                pdf.panel.VerticalScroll.Value = pdf.vScrollPos;
            }

            DisableZoomResetBtns(ref pdf);
        }

        public void ProcessZoomResetBtns(ref ProcessPdf pdf, ref PageArray pa)
        {
            if (pdf == null)
            {
                MessageBox.Show("ProcessZoomResetBtns() received a null pdf argument.", "null ProcessPdf argument:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (pa == null)
            {
                MessageBox.Show("ProcessZoomResetBtns() received a null pa argument.", "null PageArray argument:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            PgZoomData zd = null;

            pa.GetPgZoomDataRef((int)pdf.curPage - 1, ref zd);

            if (zd == null)
            {
                //MessageBox.Show(string.Format("pa.GetPgZoomDataRef({0}, ref zd) has returned a null PgZoomData.", pdf.curPage - 1), "ProcessZoomResetBtns()", MessageBoxButtons.OK, MessageBoxIcon.Error);

                // Not necessarily an error here, we did not yet do a zoom in or out.
                return;
            }

            /*
             * It is possible to have a positive zd.zoomCnt and
             * have pdf.initZoomReset being true. This happens when
             * as an example, somebody does a Full Page or a Full Width
             * after they have done a Zoom In or a Zoom Out such that the
             * zd.zoomCnt is not zero.
             * 
             * The Full Width/Page operation will cause the 
             * pdf.initZoomReset to be set but they have not yet processed
             * it, which they do by choosing a Zoom In or a Zoom Out.
             * 
             * We would want that initialization to be done here.
             * There are other scenarios where this can happen.
             * 
            */
            if ((zd.zoomCnt == 0) || (pdf.initZoomReset))
            {
                DisableZoomResetBtns(ref pdf);
            }
            else
            {
                EnableZoomResetBtns(ref pdf);
            }
        }

        public void ProcessFirstPageNav(ref ProcessPdf pdf, ref PageArray pa)
        {
            if (pdf == null)
            {
                MessageBox.Show("ProcessFirstPageNav() received a null pdf argument.", "null ProcessPdf argument:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (pa == null)
            {
                MessageBox.Show("ProcessFirstPageNav() received a null pa argument.", "null PageArray argument:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            MessageBox.Show("We are here!");

            if (pdf.pageNav)
            {
                pdf.curPage = 1;
            }
            else
            {
                // In boxed page navigation mode:
                pdf.boxedPageIndex = 0;
                pdf.curPage = pdf.boxedPageList[0];
            }

            pdf.ShowPage((int)pdf.curPage, true, ref pa);

            ProcessPageNum(ref pdf);

            ProcessZoomResetBtns(ref pdf, ref pa);
        }

        public void ProcessPrevPageNav(ref ProcessPdf pdf, ref PageArray pa)
        {
            if (pdf == null)
            {
                MessageBox.Show("ProcessPrevPageNav() received a null pdf argument.", "null ProcessPdf argument:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (pa == null)
            {
                MessageBox.Show("ProcessPrevPageNav() received a null pa argument.", "null PageArray argument:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (pdf.pageNav)
            {
                pdf.curPage -= 1;
            }
            else
            {
                // In boxed page navigation mode:
                pdf.boxedPageIndex -= 1;
                pdf.curPage = pdf.boxedPageList[pdf.boxedPageIndex];
            }

            pdf.ShowPage((int)pdf.curPage, true, ref pa);

            ProcessPageNum(ref pdf);

            ProcessZoomResetBtns(ref pdf, ref pa);
        }

        public void ProcessNextPageNav(ref ProcessPdf pdf, ref PageArray pa)
        {
            if (pdf == null)
            {
                MessageBox.Show("ProcessNextPageNav() received a null pdf argument.", "null ProcessPdf argument:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (pa == null)
            {
                MessageBox.Show("ProcessNextPageNav() received a null pa argument.", "null PageArray argument:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (pdf.pageNav)
            {
                pdf.curPage += 1;
            }
            else
            {
                // In boxed page navigation mode:
                pdf.boxedPageIndex += 1;
                pdf.curPage = pdf.boxedPageList[pdf.boxedPageIndex];
            }

            pdf.ShowPage((int)pdf.curPage, true, ref pa);

            ProcessPageNum(ref pdf);

            ProcessZoomResetBtns(ref pdf, ref pa);
        }

        public void ProcessLastPageNav(ref ProcessPdf pdf, ref PageArray pa)
        {
            if (pdf == null)
            {
                MessageBox.Show("ProcessLastPageNav() received a null pdf argument.", "null ProcessPdf argument:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (pa == null)
            {
                MessageBox.Show("ProcessLastPageNav() received a null pa argument.", "null PageArray argument:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (pdf.pageNav)
            {
                pdf.curPage = pdf.numPages;
            }
            else
            {
                // In boxed page navigation mode:
                pdf.boxedPageIndex = pdf.boxedPageList.Count - 1;
                pdf.curPage = pdf.boxedPageList[pdf.boxedPageIndex];
            }

            pdf.ShowPage((int)pdf.curPage, true, ref pa);

            ProcessPageNum(ref pdf);

            ProcessZoomResetBtns(ref pdf, ref pa);
        }

        public void ProcessFullPage(ref ProcessPdf pdf, ref PageArray pa)
        {
            if (pdf == null)
            {
                MessageBox.Show("ProcessFullPage() received a null pdf argument.", "null ProcessPdf argument:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (pa == null)
            {
                MessageBox.Show("ProcessFullPage() received a null pa argument.", "null PageArray argument:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            /*
             * This is a 'forced' dpi change, so we disable the
             * zoom reset capability.
            */

            pdf.ScaleRects("FullPage", ref pa);

            pdf.ShowPage((int)pdf.curPage, true, ref pa);

            DisableZoomResetBtns(ref pdf);
        }

        public void ProcessFullWidth(ref ProcessPdf pdf, ref PageArray pa)
        {
            if (pdf == null)
            {
                MessageBox.Show("ProcessFullWidth() received a null pdf argument.", "null ProcessPdf argument:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (pa == null)
            {
                MessageBox.Show("ProcessFullWidth() received a null pa argument.", "null PageArray argument:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            /*
             * This is a 'forced' dpi change, so we disable the
             * zoom reset capability.
            */

            pdf.ScaleRects("FullWidth", ref pa);

            pdf.ShowPage((int)pdf.curPage, true, ref pa);

            DisableZoomResetBtns(ref pdf);
        }

        public void ProcessMinMaxRestore(ref ProcessPdf pdf, ref PageArray pa)
        {
            if (pa == null)
            {
                MessageBox.Show("ProcessMinMaxRestore() received a null pa argument.", "null PageArray argument:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            /*
             * Caller has verified the pdf is not null.
             * 
             * This is called when we are minimizing, maximizing, or restoring
             * the form. We rescale the pages to make the most out of the available
             * panel area, and disable zoom reset capability.
            */

            /*
             * Restore visibility as we intercepted the call to maximize and turned this off.
             * This is the only way I could find that prevented the image from flashing
             * briefly in upper left corner of PicPanel.
            */
            if (this.WindowState == FormWindowState.Maximized)
            {
                pdf.panel.Visible = true;
            }

            if (pdf.panel.BackColor == SystemColors.Control)
            {
                if (pdf.ScaleRects("OpenFile", ref pa))
                {
                    if (pdf.numPages > 0)
                    {
                        pdf.ShowPage((int)pdf.curPage, true, ref pa);

                        DisableZoomResetBtns(ref pdf);
                    }
                }
            }
        }

        public void ProcessResize(ref ProcessPdf pdf, ref PageArray pa)
        {
            if (pdf == null)
            {
                MessageBox.Show("ProcessResize() received a null pdf argument.", "null ProcessPdf argument:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (pa == null)
            {
                MessageBox.Show("ProcessResize() received a null pa argument.", "null PageArray argument:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            //MessageBox.Show(string.Format("{0}: ProcessResize()!", pdf.id));

            int scrollBarMask = 0;

            pdf.ScaleRects("Resize", ref pa);

            if (pdf.curPageZoomDelta)
            {
                /*
                 * The Resize ended up 'growing' the image for the current
                 * page which causes a forced dpi and bitmap settings change.
                 * 
                 * We Resize ALL pages like we do for all of the other
                 * ScaleRects() operations. We will disable zoom reset
                 * capability for the current page. Odds are this applies to
                 * all pages, but it is possible for a landscape page to
                 * not encounter a forced dpi change while the other pages
                 * do.
                */
                pdf.curPageZoomDelta = false;

                DisableZoomResetBtns(ref pdf);
            }

            if (this.WindowState == FormWindowState.Maximized)
            {
                /*
                 * Restore this as we intercepted the call to maximize and turned this off.
                 * This is the only way I could find that prevented the image from flashing
                 * briefly in upper left corner of PicPanel.
                */
                pdf.panel.Visible = true;
            }

            scrollBarMask = ((pdf.panel.ClientSize.Width > pdf.pictureBox.Width ? 10 : 0) + (pdf.panel.ClientSize.Height > pdf.pictureBox.Height ? 1 : 0));
            //this.Text = string.Format("{0} scrollBarMask: {1}", pdf.id, scrollBarMask);

            switch (scrollBarMask)
            {
                case (0):

                    if ((pdf.pictureBox.Location.X > 0) || (pdf.pictureBox.Location.Y > 0))
                    {
                        Point pt = pdf.pictureBox.Location;

                        //this.Text = string.Format("{0}: {1} | Pic.X > 0 {2} | Pic.Y > 0 {3}", pdf.id, scrollBarMask, (pdf.pictureBox.Location.X > 0), (pdf.pictureBox.Location.Y > 0));

                        if (pdf.pictureBox.Location.X > 0)
                        {
                            pt.X = 0;
                        }

                        if (pdf.pictureBox.Location.Y > 0)
                        {
                            pt.Y = 0;
                        }

                        pdf.pictureBox.Location = pt;
                    }
                    break;

                case (1):
                case (10):

                    pdf.resetScrollBars = false;
                    pdf.CenterPicPanel();
                    pdf.resetScrollBars = true;

                    break;

                case (11):

                    pdf.ShowPage((int)pdf.curPage, true, ref pa);

                    break;
            };
        }

        public void ProcessCleanupFiles()
        {
            if (pdf1 != null)
            {
                pdf1.CleanupFiles();
            }

            if (pdf2 != null)
            {
                pdf2.CleanupFiles();
            }
        }

        public void ProcessPageToolStripClick(ref ProcessPdf pdf, ref PageArray pa)
        {
            if (pdf == null)
            {
                MessageBox.Show("ProcessPageToolStripClick() received a null pdf argument.", "null ProcessPdf argument:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (pa == null)
            {
                MessageBox.Show("ProcessPageToolStripClick() received a null pa argument.", "null PageArray argument:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (!pdf.pageNav)
            {
                // Switching from boxed page navigation mode to page navigation mode.

                pdf.pageNav = true;

                ProcessPageNavMode(pdf.pageNav, pdf.id);

                ProcessPageNum(ref pdf);
            }
        }

        public void ProcessBoxedPageToolStripClick(ref ProcessPdf pdf, ref PageArray pa)
        {
            if (pdf == null)
            {
                MessageBox.Show("ProcessBoxedPageToolStripClick() received a null pdf argument.", "null ProcessPdf argument:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (pa == null)
            {
                MessageBox.Show("ProcessBoxedPageToolStripClick() received a null pa argument.", "null PageArray argument:", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (pdf.pageNav)
            {
                // Switching from page navigation mode to boxed page navigation mode.

                pdf.pageNav = false;

                ProcessPageNavMode(pdf.pageNav, pdf.id);

                /*
                 * We need to set the index based on the pdf.curPage value,
                 * which can be any page, not necessarily a page with boxes.
                */
                if (pdf.curPage < pdf.boxedPageList[0])
                {
                    /*
                     * Default to the first boxed index position and move to that page.
                    */
                    pdf.boxedPageIndex = 0;
                    pdf.curPage = pdf.boxedPageList[0];
                    pdf.ShowPage((int)pdf.curPage, true, ref pa);
                }
                else if (pdf.curPage > pdf.boxedPageList[pdf.boxedPageList.Count - 1])
                {
                    /*
                     * Default to the last boxed index position and move to that page.
                    */
                    pdf.boxedPageIndex = pdf.boxedPageList.Count - 1;
                    pdf.curPage = pdf.boxedPageList[pdf.boxedPageIndex];
                    pdf.ShowPage((int)pdf.curPage, true, ref pa);
                }
                else
                {
                    /*
                     * Current page num is somewhere in the page range encompassed by the
                     * page numbers in the list. We need to determine if it is in the list,
                     * if so we use that index, we are already on that page. If not, we set
                     * for the nearest boxed index and move to that page.
                    */
                    if (pdf.boxedPageList.Contains(pdf.curPage))
                    {
                        pdf.boxedPageIndex = pdf.boxedPageList.IndexOf(pdf.curPage);
                    }
                    else
                    {
                        for (int i = 1; i < pdf.boxedPageList.Count; i++)
                        {
                            if (pdf.curPage < pdf.boxedPageList[i])
                            {
                                // We will autoselect the closest boxed page:
                                if ((pdf.boxedPageList[i] - pdf.curPage) < (pdf.curPage - pdf.boxedPageList[i - 1]))
                                {
                                    pdf.boxedPageIndex = i;
                                }
                                else
                                {
                                    pdf.boxedPageIndex = i - 1;
                                }

                                break;
                            }
                        }

                        pdf.curPage = pdf.boxedPageList[pdf.boxedPageIndex];
                        pdf.ShowPage((int)pdf.curPage, true, ref pa);
                    }
                }

                ProcessPageNum(ref pdf);
            }
        }

        static public Rectangle GetRectangle(Point p1, Point p2)
        {
            return new Rectangle(Math.Min(p1.X, p2.X), Math.Min(p1.Y, p2.Y),
                Math.Abs(p1.X - p2.X), Math.Abs(p1.Y - p2.Y));
        }

        public void ProcessSingleScrnCLick()
        {
            /*
             * User is switching from a split screen view to a single screen
             * view. We end up collapsing the pdf2 panel. The pdf1 panel always
             * exists in an uncollapsed state, it is the pdf2 panel that collapses
             * and uncollapses as the user clicks the screen split buttons.
             * 
             * When the pdf1 panel and the pdf2 panel are both empty:
             * 
             *      We end up with one empty pdf1 panel.
             *  
             * When the pdf1 panel is not empty and the pdf2 panel is empty:
             * 
             *      We end up with one pdf1 panel rescaled to the new panel size.
             *
             * When the pdf1 panel is not empty and the pdf2 panel is not empty:
             * 
             *      We end up with one pdf1 panel rescaled to the new panel size.
             *      The pdf1 panel currently has precedence in this situation.
             *
             * When the pdf1 panel is empty and the pdf2 panel is not empty:
             * 
             *      We end up with one pdf1 panel rescaled to the new panel size
             *      and it is displaying what the pdf2 panel was displaying.
             *      
             * *****************************************************
             * 
             * IMPORTANT FUTURE NOTES REGARDING PDF2 PANEL COLLAPSE:
             * 
             * WE WILL NEED TO CHECK IF THE PDF2 FILE IS COUPLED WITH THE 
             * TOOL'S CALCULATION LOGIC. IF IT IS, WE NEED TO PROMPT THE
             * USER CONCERNING THIS AS INFO/CALCS CURRENLY BUFFERED/ETC.
             * WILL BE LOST. THIS WILL REQUIRE MORE CONSIDERATION, JUST A
             * NOTE FOR FUTURE DEVELOPMENT EFFORTS.
             * 
             * *****************************************************
            */
            int pdfMask;

            pdfMask = (pdf1 == null ? 0 : 10) + (pdf2 == null ? 0 : 1);

            splitScreen = false;

            splitContainer1.Panel2Collapsed = true;

            switch (pdfMask)
            {
                case (0):

                    // pdf1 and padf2 are both empty:
                    singleScrnBtn.Enabled = false;
                    singleScreenToolStripMenuItem.Enabled = false;

                    splitVertBtn.Enabled = false;
                    splitScreenVerticallyToolStripMenuItem.Enabled = false;

                    splitHorizBtn.Enabled = false;
                    splitScreenHorizontallyToolStripMenuItem.Enabled = false;

                    break;

                case (1):

                    // pdf1 is empty, padf2 is not empty:
                    singleScrnBtn.Enabled = false;
                    singleScreenToolStripMenuItem.Enabled = false;

                    DisableZoomResetBtns(ref pdf2);

                    CopyPdf2ToPdf1(true);

                    break;

                case (10): // pdf1 is not empty, padf2 is empty:
                case (11): // pdf1 is not empty, padf2 is not empty:
                    
                    singleScrnBtn.Enabled = false;
                    singleScreenToolStripMenuItem.Enabled = false;

                    splitVertBtn.Enabled = true;
                    splitScreenVerticallyToolStripMenuItem.Enabled = true;

                    splitHorizBtn.Enabled = true;
                    splitScreenHorizontallyToolStripMenuItem.Enabled = true;

                    /*
                     * Going to rescale pdf1 to larger panel and will reset
                     * the 'zoom reset' buttons if enabled.
                    */
                    if (pdf1.ScaleRects("OpenFile", ref pa1))
                    {
                        if (pdf1.numPages > 0)
                        {
                            pdf1.ShowPage((int)pdf1.curPage, true, ref pa1);

                            DisableZoomResetBtns(ref pdf1);
                        }
                    }

                    if (pdfMask == 11)
                    {
                        // Make sure we do not cleanup pdf and xml files if being used by pdf1.
                        if (pdf2.ExtPath != pdf1.ExtPath)
                        {
                            pdf2.CleanupFiles();
                        }

                        pdf2.ClearBmaps();

                        DisableZoomResetBtns(ref pdf2);

                        pdf2 = null;
                        pa2 = null;
                    }

                    break;
            }
        }

        public void ProcessSplitHorScrnCLick()
        {
            /*
             * If the screen is already split vertically: We will just do the horizontal
             * split with what the panels may or may not contain. If they are displaying
             * a pdf we will rescale each page's rectangles to account for the dimensional
             * change.
             * 
             * If the screen is not currently split: We will rescale the pdf1 rectangles
             * if the pdf1 is not null and also copy any pdf1 content to the pdf2 panel
             * rescaling each page's rectangles for that panel.
            */

            /*
             * **********************************************************
             * 
             * We can do these lines of code regardless of split screen mode.
            */
            splitHorizBtn.Enabled = false;
            splitScreenHorizontallyToolStripMenuItem.Enabled = false;

            copyPaneToolStripMenuItem2.Text = "Copy Top Pane";

            splitContainer1.Orientation = Orientation.Horizontal;
            splitContainer1.SplitterDistance = (int)(splitContainer1.ClientSize.Height * 0.5);

            /*
             * **********************************************************
            */

            if (splitScreen)
            {
                splitVertBtn.Enabled = true;
                splitScreenVerticallyToolStripMenuItem.Enabled = true;

                if (pdf2 != null)
                {
                    /*
                     * Rescale all pdf2 page boxes to reflect new panel dimension/size.
                     * The pdf2 panel may or may not be displaying the same pdf
                     * as being displayed in the pdf1 panel.
                    */
                    if (pdf2.ScaleRects("OpenFile", ref pa2))
                    {
                        if (pdf2.numPages > 0)
                        {
                            pdf2.ShowPage((int)pdf2.curPage, true, ref pa2);

                            DisableZoomResetBtns(ref pdf2);
                        }
                    }
                }
            }
            else
            {
                splitScreen = true;

                /*
                 * Form view will be split horizontally. We will set the 2
                 * display panels to each consume 50% of the viewable Form
                 * area. Each pdf will be re-scaled to fit the new reduced
                 * area if a pdf is being displayed.
                */
                singleScrnBtn.Enabled = true;
                singleScreenToolStripMenuItem.Enabled = true;

                splitContainer1.Panel2Collapsed = false;

                if (pdf1 != null)
                {
                    /*
                     * 'true' means we want the same pdf1 page displayed in the
                     * pdf2 panel, same button statuses.
                    */
                    CopyPdf1ToPdf2(true);
                }
            }

            if (pdf1 != null)
            {
                copyPaneToolStripMenuItem2.Enabled = true;

                /*
                 * Rescale all page boxes to reflect new panel dimension/size.
                 * 
                 * ScaleRects("OpenFile", ref pa1) signals a reset of the PgZoomData
                 * for each page since we are moving to a new panel size, so for now we
                 * will disable the zoom reset buttons. That signalling is caught by
                 * the zoom in and zoom out routines.
                */
                if (pa1 != null)
                {
                    if (pdf1.ScaleRects("OpenFile", ref pa1))
                    {
                        if (pdf1.numPages > 0)
                        {
                            pdf1.ShowPage((int)pdf1.curPage, true, ref pa1);

                            DisableZoomResetBtns(ref pdf1);
                        }
                    }
                }
            }
        }

        public void ProcessSplitVerScrnCLick()
        {
            /*
             * If the screen is already split horizontally: We will just do the vertical
             * split with what the panels may or may not contain. If they are displaying
             * a pdf we will rescale each page's rectangles to account for the dimensional
             * change.
             * 
             * If the screen is not currently split: We will rescale the pdf1 rectangles
             * if the pdf1 is not null and also copy any pdf1 content to the pdf2 panel
             * rescaling each page's rectangles for that panel.
            */

            /*
             * **********************************************************
             * 
             * We can do these lines of code regardless of split screen mode.
            */
            splitVertBtn.Enabled = false;
            splitScreenVerticallyToolStripMenuItem.Enabled = false;

            copyPaneToolStripMenuItem2.Text = "Copy Left Pane";

            splitContainer1.Orientation = Orientation.Vertical;
            splitContainer1.SplitterDistance = (int)(splitContainer1.ClientSize.Width * 0.5);

            /*
             * **********************************************************
            */ 

            if (splitScreen)
            {
                splitHorizBtn.Enabled = true;
                splitScreenHorizontallyToolStripMenuItem.Enabled = true;

                if (pdf2 != null)
                {
                    /*
                     * Rescale all pdf2 page boxes to reflect new panel dimension/size.
                     * The pdf2 panel may or may not be displaying the same pdf
                     * as being displayed in the pdf1 panel.
                    */
                    if (pdf2.ScaleRects("OpenFile", ref pa2))
                    {
                        if (pdf2.numPages > 0)
                        {
                            pdf2.ShowPage((int)pdf2.curPage, true, ref pa2);

                            DisableZoomResetBtns(ref pdf2);
                        }
                    }
                }
            }
            else
            {
                splitScreen = true;

                /*
                 * Form view will be split vertically. We will set the 2
                 * display panels to each consume 50% of the viewable Form
                 * area. Each pdf will be re-scaled to fit the new reduced
                 * area if a pdf is being displayed.
                */
                singleScrnBtn.Enabled = true;
                singleScreenToolStripMenuItem.Enabled = true;

                splitContainer1.Panel2Collapsed = false;

                if (pdf1 != null)
                {
                    /*
                     * 'true' means we want the same pdf1 page displayed in the
                     * pdf2 panel, same button statuses.
                    */
                    CopyPdf1ToPdf2(true);
                }
            }

            if (pdf1 != null)
            {
                copyPaneToolStripMenuItem2.Enabled = true;

                /*
                 * Rescale all page boxes to reflect new panel dimension/size.
                 * 
                 * ScaleRects("OpenFile", ref pa1) signals a reset of the PgZoomData
                 * for each page since we are moving to a new panel size, so for now we
                 * will disable the zoom reset buttons. That signalling is caught by
                 * the zoom in and zoom out routines.
                */
                if (pa1 != null)
                {
                    if (pdf1.ScaleRects("OpenFile", ref pa1))
                    {
                        if (pdf1.numPages > 0)
                        {
                            pdf1.ShowPage((int)pdf1.curPage, true, ref pa1);

                            DisableZoomResetBtns(ref pdf1);
                        }
                    }
                }
            }
        }

        public void ProcessMouseDown(ref ProcessPdf pdf,  Point location)
        {
            /*
             * Caller has ensured that pdf is not null and it is a left
             * mouse button event.
            */
            pdf.mDown = location;
            pdf.mLassoDown = true;
        }

        public void ProcessMouseMove(ref ProcessPdf pdf, Point location)
        {
            /*
             * Caller has ensured that pdf is not null and it is a left
             * mouse button event.
            */
            if (pdf.mLassoDown)
            {
                pdf.mLassoMove = true;

                using (Graphics g = pdf.pictureBox.CreateGraphics())
                {
                    Pen penSel = new Pen(Color.Black, 2f);
                    penSel.DashStyle = System.Drawing.Drawing2D.DashStyle.Dot;

                    pdf.pictureBox.Refresh();
                    pdf.lassoRect = GetRectangle(pdf.mDown, location);
                    g.DrawRectangle(penSel, pdf.lassoRect);
                }
            }
        }

        public void ProcessMouseClick(ref ProcessPdf pdf, ref PageArray pa, Point location)
        {
            /*
             * Caller has ensured that pdf and pa are not null, and pdf.mLassoMove
             * is false, and it is a left mouse button event.
            */
            pdf.mLassoDown = false;

            List<Field> fa = null;

            pa.GetFieldArrayRef((int)(pdf.curPage - 1), ref fa);

            if (fa != null)
            {
                for (int i = 0; i < fa.Count; i++)
                {
                    if (fa[i].sbox.Contains(location))
                    {
                        //MessageBox.Show(string.Format("{0} was clicked!", fa[i].ref_name));

                        // For now, we unselect selected boxes and vice versa.
                        fa[i].isSelected = !fa[i].isSelected;

                        pdf.centerPic = false;

                        pdf.ShowPage((int)pdf.curPage, false, ref pa);

                        pdf.centerPic = true;

                        break;
                    }
                }
            }
        }

        public void ProcessMouseUp(ref ProcessPdf pdf, ref PageArray pa)
        {
            /*
             * Caller has ensured that pdf and pa are not null, and it is a
             * left mouse button event.
            */
            if (pdf.mLassoMove)
            {
                List<Field> fa = null;
                int cnt = 0;

                pa.GetFieldArrayRef((int)(pdf.curPage - 1), ref fa);

                if (fa != null)
                {
                    for (int i = 0; i < fa.Count; i++)
                    {
                        if (fa[i].sbox.IntersectsWith(pdf.lassoRect))
                        {
                            //MessageBox.Show(string.Format("{0} was clicked!", fa[i].ref_name));

                            cnt += 1;

                            // For now, we unselect selected boxes and vice versa.
                            fa[i].isSelected = !fa[i].isSelected;
                        }
                    }

                    if (cnt > 0)
                    {
                        pdf.centerPic = false;

                        pdf.ShowPage((int)pdf.curPage, false, ref pa);

                        pdf.centerPic = true;
                    }
                    else
                    {
                        // Killing the rect like this rather than via pdf.ShowPage().
                        pdf.pictureBox.Refresh();
                    }
                }

                //pdf.lassoRect = new Rectangle(0, 0, 0, 0);
                pdf.mLassoMove = false;
                pdf.mLassoDown = false;
            }
        }
    }
}